@@ -4,7 +4,7 @@ use 5.010;
 use strict;
 use warnings;
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 use strict;
 use warnings;
@@ -97,9 +97,9 @@ sub	config2perl {
 	# we can not be sure if this <% xyz %> is part of the content or a forgotten templated variable
 	if( $contents =~ /\Q${tvop}\E!(:?(:?begin-verbatim-section)|(:?end-verbatim-section))%*>/ ){ warn "--begin content:\n".$contents."\n--end content.\n".__PACKAGE__.'::config2perl()'." (line ".__LINE__.") : warning, there may still be remains of templated variables in the specified content, see above what remained after all template variables substitutions were done." }
 
-	# secondly, remove the VERBATIM multiline sections and transform them
-	# comments inside the verbatim section will NOT BE touched.
-	# the only thing touched was the templated variables earlier
+	# secondly, remove the VERBATIM multiline sections and transform them.
+	# Comments inside the verbatim section will NOT BE touched.
+	# The only thing touched was the templated variables earlier
 	# it substitutes each verbatim section with a code
 	# then does the comments and then replaces the code with the verbatim section at the very end
 	my @verbs;
@@ -201,7 +201,7 @@ Config::JSON::Enhanced - JSON-based config with C/Shell-style comments, verbatim
 
 =head1 VERSION
 
-Version 0.06
+Version 0.07
 
 =head1 SYNOPSIS
 
@@ -209,27 +209,38 @@ This module provides subroutine C<config2perl()> for parsing configuration conte
 from files or strings,  based on, what I call, "enhanced JSON" (see section
 L<ENHANCED JSON FORMAT> for more details). Briefly, it is standard JSON which allows:
 
-
 =over 2
 
-=item * C<C>-style, C<C++>-style or C<shell>-style comments.
+=item * C<C>-style, C<C++>-style, C<shell>-style or custom comments.
 
-=item * template variables (e.g. C<E<lt>% appdir %E<gt>>) which are substituted with user-specified data.
+=item * Template-style variables (e.g. C<E<lt>% appdir %E<gt>>)
+which are substituted with user-specified data during parsing.
 
-=item * verbatim sections which are a sort of here-doc for JSON
-which may be spanning multiple
-lines and contained single and double quotes are not required to be escaped.
-This enhances the readbility of long JSON which may contain, in my case,
-long shell scripts with lots of quotes and newlines.
+=item * Verbatim sections which are a sort of here-doc for JSON,
+allowing strings to span multiple
+lines, to contain single and double quotes unescaped,
+to contain template-style variables.
 
 =back
 
-It has been tested with unicode data (see C<t/25-config2perl-complex-utf8.t>)
+This module was created because I needed to include
+long shell scripts containing lots of quotes and newlines,
+in a configuration file which started as JSON.
+
+The process is simple: so-called "enhanced JSON" is parsed
+by L<config2perl>. Comments are removed, variables are
+substituted, verbatim sections become one line again
+and standard JSON is created. This is parsed with
+L<JSON> (via L<Data::Roundtrip::json2perl>) to
+produce a Perl data structure which is returned.
+
+It has been tested with unicode data
+(see C<t/070-config2perl-complex-utf8.t>)
 with success. But who knows ?!?!
 
 Here is an example:
 
-    use Config::JSON::Enhanced qw/config2perl/;
+    use Config::JSON::Enhanced;
 
     # simple "enhanced" JSON with comments in 3 styles: C,shell,CPP
     my $configdata = <<'EOJ';
@@ -250,10 +261,48 @@ Here is an example:
     # {"a" : "abc","b" : [1,2,3], "c" : 12}
 
 
+    # this "enhanced" JSON demonstrates the use of variables
+    # which will be substituted during the transformation to
+    # standard JSON with user-specified data.
+    # Notice that the opening and closing tags enclosing variable
+    # names can be customised using the 'tags' input parameter,
+    # so as to avoid clashes.
+    my $configdata = <<'EOJ';
+     {
+       "d" : [1,2,<% tempvar0 %>],
+       "configfile" : "<%SCRIPTDIR%>/config/myapp.conf",
+       "username" : "<% username %>"
+        }
+     }
+    EOJ
+    my $perldata = config2perl({
+        'string' => $configdata,
+        'commentstyle' => "C,shell,CPP",
+        # optionally customise the tags enclosing the variables
+        #'tags' => ['<%', '%>'],
+        # user-specified data to replace the variables in
+        # the "enhanced" JSON above:
+        'variable-substitutions' => {
+            'tempvar0' => 42,
+            'username' => getlogin(),
+            'SCRIPTDIR' => $FindBin::Bin,
+        },
+    });
+    die "call to config2perl() has failed" unless defined $perldata;
+    # the standard JSON
+    # (notice how all variables in <%...%> are now replaced):
+    # {"d" : [1,2,42],
+    #  "username" : "yossarian",
+    #  "configfile" : "/home/yossarian/B52/config/myapp.conf"
+    # }
+
+
     # this "enhanced" JSON demonstrates "verbatim sections"
     # the puprose of which is to make more readable JSON strings
-    # by allowing them spanning over multiple lines
-    # and no need for escaping double quotes.
+    # by allowing them to span over multiple lines.
+    # There is also no need for escaping double quotes.
+    # template variables (like above) will be substituted
+    # There will be no comments removal from the verbatim sections.
     my $configdata = <<'EOJ';
      {
       "a" : <%begin-verbatim-section%>
@@ -261,8 +310,6 @@ Here is an example:
       string
       "quoted text" and 'quoted like this also'
       will be retained in the string escaped.
-      Comments like /* this */ or # this comment
-      will be removed.
       White space from beginning and end will be chomped.
  
       <%end-verbatim-section%>
@@ -280,47 +327,17 @@ Here is an example:
     # {"a" :
     #   "This is a multiline\nstring\n\"quoted text\" and 'quoted like
     #   this also'\nwill be retained in the string escaped.\nComments
-    #   like  or # this comment\nwill be removed.\nWhite space from
+    #   will not be removed.\nWhite space from
     #   beginning and end will be chomped.",
     #  "b" : 123
     # };
 
 
-    # this "enhanced" JSON demonstrates the use of variables
-    # which will be substituted during the transformation to
-    # standard JSON with user-specified data.
-    my $configdata = <<'EOJ';
-     {
-       "d" : [1,2,<% tempvar0 %>],
-       "configfile" : "<%SCRIPTDIR%>/config/myapp.conf",
-       "username" : "<% username %>"
-        }
-     }
-    EOJ
-    my $perldata = config2perl({
-        'string' => $configdata,
-        'commentstyle' => "C,shell,CPP",
-        # user-specified data to replace the variables in
-        # the "enhanced" JSON above:
-        'variable-substitutions' => {
-            'tempvar0' => 42,
-            'username' => getlogin(),
-            'SCRIPTDIR' => $FindBin::Bin,
-        },
-    });
-    die "call to config2perl() has failed" unless defined $perldata;
-    # the standard JSON
-    # (notice how all variables in <%...%> are now replaced):
-    # {"d" : [1,2,42],
-    #  "username" : "yossarian",
-    #  "configfile" : "/home/yossarian/B52/config/myapp.conf"
-    # }
-
 =head1 EXPORT
 
 =over 4
 
-=item * C<config2perl>
+=item * C<config2perl> is exported by default.
 
 =back
 
@@ -330,12 +347,13 @@ Here is an example:
 =head2 C<config2perl>
 
   my $ret = config2perl($params);
+  die unless defined $ret;
 
 Arguments:
 
 =over 4
 
-=item * C<$params>
+=item * C<$params> : a hashref of input parameters.
 
 =back
 
@@ -343,26 +361,33 @@ Return value:
 
 =over 4
 
-=item * C<$ret> on success or C<undef> on failure.
+=item * the parsed content as a Perl data structure
+on success or C<undef> on failure.
 
 =back
 
-Given I<Enhanced JSON> content, this sub removes comments
-(as per input parameters),
-replaces all template strings, if any, and then parses
-what remains as standard JSON into a Perl data structure which
-is returned.
+Given input content in L<ENHANCED JSON FORMAT>, this sub removes comments
+(as per preferences via input parameters),
+replaces all template variables, if any,
+compacts L<Verbatim Sections>, if any, into a single-line
+string and then parses
+what remains as standard JSON into a Perl data structure
+which is returned to caller. JSON parsing is done with
+L<Data::Roundtrip::json2perl>, which uses L<JSON>.
 
-JSON content to-be-parsed can be specified with one of the following
-keys in the input parameters hashref (C<$params>):
-
-Comments outside of JSON fields will always be removed.
+Comments outside of JSON fields will always be removed,
+otherwise JSON can not be parsed.
 
 Comments inside of JSON fields, keys, values, strings etc.
 will not be removed unless input parameter C<remove-comments-in-strings>
-is set to 1.
+is set to 1 by the caller.
 
-Comments inside L<Verbatim Sections> will never be removed.
+Comments (or what looks like comments with the current input parameters)
+inside L<Verbatim Sections> will never be removed.
+
+The input content to-be-parsed can be specified
+with one of the following input parameters (entries in the
+C<$params>):
 
 =over 4
 
@@ -381,7 +406,7 @@ Additionally, input parameters can contain the following keys:
 
 =item * C<commentstyle> : specify what comment style(s) to be expected
 in the input content (if any) as a B<comma-separated string>. For example
-C<'C,CPP,shell,custom(<<)(>>),custom(REM)()'>.
+C<'C,CPP,shell,custom(E<lt>E<lt>)(E<gt>E<gt>),custom(REM)()'>.
 These are the values it understands:
 
 =over 2
@@ -398,12 +423,13 @@ For example C</* I am a comment */>, C<// I am a comment to the end of line>.
 For example, C<# I am a comment to the end of line>.
 
 =item * C<custom> : comments are enclosed (or preceded) by custom,
-user-specified strings. The form is C<custom(OPENING)(CLOSING)>.
-C<OPENING> is required and C<CLOSING> is optional meaning that
+user-specified tags. The form is C<custom(OPENINGTAG)(CLOSINGTAG)>.
+C<OPENINGTAG> is required. C<CLOSINGTAG> is optional meaning that
 the comment extends to the end of line (just like C<shell> comments).
 For example C<custom(E<lt>E<lt>)(E<gt>E<gt>)> or 
 C<custom({{)(})> or C<custom(REM)()> or C<custom(E<lt>E<lt>E<lt>E<lt>)(E<gt>E<gt>)>.
-C<OPENING> and C<CLOSING> do not need to be of the same character length as it is
+C<OPENINGTAG> and C<CLOSINGTAG> do not need to be of
+the same character length as it is
 obvious from the previous example.
 
 =back
@@ -424,13 +450,19 @@ under the parameters key C<variable-substitutions>, for example:
   });
 
 Variable substitution will be performed in both
-keys and values of the input JSON.
+keys and values of the input JSON, including L<Verbatim Sections>.
+
+=item * C<remove-comments-in-strings> : by default no attempt
+to remove what-looks-like-comments from JSON strings
+(both keys and values). However, if this flag is set to
+C<1> anything that looks like comments (as per the 'C<commentstyle>'
+parameter) will be removed from inside all JSON strings
+(keys or values) unless they were part of verbatim section.
+
+This does not apply for the content verbatim sections.
+What looks like comments to us, inside verbatim sections
+will be left intact.
 
-=item * C<remove-comments-in-strings> : by default JSON strings
-(both keys and values) are keeping all their content,
-even if it looks like it contains comments which they are usually
-removed elsewhere. By setting this flag to 1, anything that looks
-like a comment we must understand (as per the parameter C<commentstyle>).
 For example consider the JSON string C<"hello/*a comment*/">
 (which can be a key or a value). If C<remove-comments-in-strings> is
 set to 1, then the JSON string will become C<hello>. If set to
@@ -439,8 +471,10 @@ set to 1, then the JSON string will become C<hello>. If set to
 =item * C<tags> : specify the opening and closing tags for template
 variables and verbatim section as an ARRAYref of exactly 2 items (the
 opening and the closing tags). By default the opening tag is C<E<gt>%>
-and the closing tag is C<%E<lt>>. By setting C<tags => [ '[::', '::]' ]>
-you must declare templated variables like this: C<{:: var1 ::]> and
+and the closing tag is C<%E<lt>>.
+
+If you set C<tags => [ '[::', '::]' ]>
+then your template variables should look like this: C<{:: var1 ::]> and
 verbatim sections like this: C<[:: begin-verbatim-section ::]>.
 
 =item * C<debug> : set this to a positive integer to increase verbosity
@@ -454,6 +488,7 @@ of B<what I call> I<enhanced JSON>.
 C<config2perl> returns the parsed content as a Perl data structure
 on success or C<undef> on failure.
 
+
 =head1 ENHANCED JSON FORMAT
 
 This is JSON with added reasonable, yet completely ad-hoc, enhancements
@@ -485,7 +520,7 @@ comment.
 =back
 
 =item * B<Template variables support> : template-style
-variables in the form of C<E<lt>%HOMEDIR%E<gt>>
+variables in the form of C<E<lt>% HOMEDIR %E<gt>>
 will be substituded with values specified by the
 user during parsing. Note that variable
 names are case sensitive, they can contain spaces, hyphens etc.,
@@ -496,12 +531,14 @@ also notice the absence of any spaces before C<abc> and after C<xyz>).
 
 The tags for denoting a template variable
 are controled by the 'C<tags>' parameter to the sub L<config2perl>.
-C<E<lt>%> and C<%E<gt>> are the defaults.
+Defaults are C<E<lt>%> and C<%E<gt>>.
 
 =item * B<Verbatim Sections> : similar to here-doc, this feature allows
 for string values to span over multiple lines and to contain
 un-escpaed quotes. This is useful if you want a JSON value to
-contain a shell script, for example.
+contain a shell script, for example. Verbatim sections can
+also contain template variables which will be substituted. No
+comment will be removed.
 
 =item * Unfortunately, there is not support for ignoring B<superfluous commas> in JSON,
 in the manner of glorious Perl.
@@ -514,16 +551,18 @@ A B<Verbaitm Section> in this ad-hoc, so-called I<Enhanced JSON> is content
 enclosed within C<E<lt>%begin-verbatim-section%E<gt>>
 and C<E<lt>%end-verbatim-section%E<gt>> tags.
 A verbatim section's content may span multiple lines (which when converted to JSON will preserve them
-by escaping), can contain comments (see the beginning of this section) and can
+by escaping. e.g. by replacing them with 'C<\n>') and can
 contain template variables to be substituted with user-specified data.
+All single and double quotes can be left un-escaped, the program will
+escape them (hopefully correctly!).
 
 The content of Verbatim Sections will have all its
-template variables substituted. But its comments will
+template variables substituted. Comments will
 be left untouched.
 
 The tags for denoting the opening and closing a verbatim section
 are controled by the 'C<tags>' parameter to the sub L<config2perl>.
-C<E<lt>%> and C<%E<gt>> are the defaults.
+Defaults are C<E<lt>%> and C<%E<gt>>.
 
 Here is an example of enhanced JSON which contains comments, a verbatim section
 and template variables:
@@ -533,8 +572,9 @@ and template variables:
     "long bash script" : ["/usr/bin/bash",
   /* This is a verbatim section */
   <%begin-verbatim-section%>
-    # save current dir
+    # save current dir, this comment remains
     pushd . &> /dev/null
+    # following quotes will be escaped
     echo "My 'appdir' is \"<%appdir%>\""
     echo "My current dir: " $(echo $PWD) " and bye"
     # go back to initial dir
@@ -565,15 +605,18 @@ and template variables:
     "expected result" => 42,
     "long bash script" => [
       "/usr/bin/bash",
-      "# save current dir\npushd . &> /dev/null\necho \"My 'appdir'
-       is \\\"/home/andreas/PROJECTS/CPAN/Config-JSON-Enhanced/t\\\"\"\necho
-       \"My current dir: \" \$(echo \$PWD) \" and bye\"\n# go back to
-       initial dir\npopd &> /dev/null"
+      "# save current dir, this comment remains\npushd . &> /dev/null\n
+       # following quotes will be escaped\necho \"My 'appdir' is
+       \\\"/home/babushka/Config-JSON-Enhanced/t\\\"\"\n
+       echo \"My current dir: \" \$(echo \$PWD) \" and bye\"\n# go back to
+       initial dir, this comment remains\npopd &> /dev/null"
     ]
   };
 
 A JSON string can contain comments which
-you may want to retain.
+you may want to retain (note:
+comments filtering will not apply to verbatim sections).
+
 For example if the
 content is a unix shell script it is
 possible to contain comments like C<# comment>.
@@ -581,13 +624,13 @@ These will be removed along with all other comments
 in the entire JSON input if you are using
 C<shell> style comments. Another problem
 is when JSON string contains comment opening
-or closing strings. For example consider this
+or closing tags. For example consider this
 cron spec : C<*/15 * * * *> which contains
 the closing string of a C-style comment and
-will mess everything.
+will cass a big mess.
 
 You have two options
-in order to retain these comments:
+in order to deal with this problem:
 
 =over 2
 
@@ -596,16 +639,24 @@ parameter to sub L<config2perl> to 0. This will
 keep ALL comments in all strings (both keys and values).
 This is a one-size-fits-all solution and it is not ideal.
 
-=item * The best solution is to change the comment style
+=item * The B<best solution> is to change the comment style
 of the input, so called Enhanced, JSON to something
 different to the comments you are trying to keep in your
 strings. So, for example, if you want to retain the comments
 in a unix shell script then use C as the comment style for
-the JSON. Note that it is possible (since version 0.03) to
-use custom comments for the JSON. This greatly increases
+the JSON.
+
+Note that it is possible (since version 0.03) to
+use custom tags for comments. This greatly increases
 your chances to make L<config2perl> understand what
 comments you want to keep as part of your data.
 
+For example, make your comments like C<[::: comment :::]>
+or even like C<E<lt>!-- comment --E<gt>> using
+C<'commentstyle' =E<gt> 'custom([:::)(:::])'>
+and C<'commentstyle' =E<gt> 'custom(E<lt>!--)(--E<gt>)'>,
+respectively.
+
 =back
 
 =head1 TIPS
@@ -618,7 +669,7 @@ the same character sequence for these tags and avoid clashes
 and unexpected substitutions. C<E<lt>%> and C<%E<gt>> are the default
 tags.
 
-Similarly, custom comment style can be employed if your
+Similarly, C<custom> comment style can be employed if your
 JSON strings contain something that looks like comments
 and you want to avoid their removal.
 
@@ -630,14 +681,13 @@ expression for
 extracting quoted strings is employed. It finds anything
 within two double quotes. It tries to handle escaped quotes within
 quoted strings.
-This may be buggy or may not
+This regex may be buggy or may not
 be complex enough to handle all corner cases. Therefore, it is
 possible that setting parameter C<remove-comments-in-strings> to 1
 to sub L<config2perl> to cause unexpected results. Please
 report these cases, see L<SUPPORT>.
 
 
-
 =head1 AUTHOR
 
 Andreas Hadjiprocopis, C<< <bliako at cpan.org> >>
